<?php

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Custom_Related_Products
 * @subpackage Custom_Related_Products/public
 * @author     markhf
 */
class Custom_Related_Products_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        add_action('wp_footer', array($this, 'wt_process_slider_script'), 99999);
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {
        //wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/custom-related-products-public.css', array(), $this->version, 'all');
		$slider_state	 = get_option( 'custom_related_products_slider','enable' );
		if('enable' == $slider_state){
                    wp_enqueue_style('bxslider-css', plugin_dir_url(__FILE__) . 'css/jquery.bxslider.min.css', array(), $this->version, 'all');
			wp_enqueue_style('swiper-css', plugin_dir_url(__FILE__) . 'css/swiper.min.css', array(), $this->version, 'all');
		}
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        //wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/custom-related-products-public.js', array('jquery'), $this->version, false);
		$slider_state	 = get_option( 'custom_related_products_slider','enable' );
		if('enable' == $slider_state){
                    wp_enqueue_script('bxslider-js', plugin_dir_url(__FILE__) . 'js/jquery.bxslider.min.js', array('jquery'), $this->version, false);
                    wp_enqueue_script('wt-swiper-js', plugin_dir_url(__FILE__) . 'js/swiper.js', array('jquery'), $this->version, false);	
		}
    }

    public function crp_filter_related_products($args) {
        
        global $post;
        $related = get_post_meta($post->ID, '_crp_related_ids', true);

        $related_categories_ids = get_post_meta($product_id, '_crp_related_product_cats', true);
		$related_tags_ids = get_post_meta($product_id, '_crp_related_product_tags', true);

        $related = $this->get_product_category_ids($related_categories_ids,$related);
		$related = $this->get_product_tag_ids($related_tags_ids,$related);

        $disable = get_option('custom_related_products_disable');
        if (isset($related) && !empty($related) && !empty($disable)) { 
            $args['post__in'] = $related;
        } elseif(empty($disable)) { 
            $args = $args;
        }

        return $args;
    }

    public function crp_woocommerce_locate_template($template, $template_name, $template_path) {
       
       
        global $woocommerce;
        $_template = $template;

        if (!$template_path) {
            $template_path = $woocommerce->template_url;
        }

        $plugin_path = CRP_PLUGIN_DIR_PATH . '/woocommerce/';
        
        $template = locate_template(
                array(
                    $template_path . $template_name,
                    $template_name
                )
        );
       
        $overide_theme_rp = ( get_option('custom_related_products_overide_theme_rp', 'enable') == 'enable') ? true : false;
        $override_theme_template = apply_filters( 'wt_crp_override_theme_template', $overide_theme_rp );
        if ( ( !$template ||  $override_theme_template ) && file_exists($plugin_path . $template_name) ) {
            $template = $plugin_path . $template_name;
        }
        // Use default template

        if (!$template) {
            $template = $_template;
        }
        // Return what we found
        return $template;
    }
    function wt_woocommerce_output_related_products() {
        global $product;

		if ( ! $product ) {
			return;
		}
        $related_products = wc_get_related_products( $product->get_id() );
        wc_get_template( 'single-product/related.php', array( 'related_products' => $related_products ), plugin_dir_path( __FILE__ ) . 'woocommerce/', '' );
    }
    
    function crp_display_ids( $result, $product_id ) {
        
        $related_ids = get_post_meta( $product_id, '_crp_related_ids', true );

        $related_categories_ids = get_post_meta($product_id, '_crp_related_product_cats', true);
		
		$related_tag_ids = get_post_meta($product_id, '_crp_related_product_tags', true);

        $related_ids = $this->get_product_category_ids($related_categories_ids,$related_ids);
		$related_ids = $this->get_product_tag_ids($related_tag_ids,$related_ids);

        return empty( $related_ids ) ? $result : true;
		
     }
        
        
	function crp_remove_taxonomy( $result, $product_id ) {
            
        $related = get_post_meta( $product_id, '_crp_related_ids', true );
        
        $related_categories_ids = get_post_meta($product_id, '_crp_related_product_cats', true);
		
		$related_tag_ids = get_post_meta($product_id, '_crp_related_product_tags', true);

        $related = $this->get_product_category_ids($related_categories_ids,$related);
		
		$related = $this->get_product_tag_ids($related_tag_ids,$related);

        if ( ! empty( $related ) ) {
            return false;
        } else {
            return $result;
        }
    }
    
    function crp_related_products_query( $query, $product_id ) {
        $modify_default_mode_query = apply_filters( 'wt_crp_modify_default_mode_query', false );
        $working_mode = Custom_Related_Products::get_current_working_mode();
        if( $working_mode == 'default' && !$modify_default_mode_query ) {
            return $query;
        }
        $related = get_post_meta( $product_id, '_crp_related_ids', true );

        $related_categories_ids = get_post_meta($product_id, '_crp_related_product_cats', true);

        $related = $this->get_product_category_ids($related_categories_ids,$related);
		
		$related_tag_ids = get_post_meta($product_id, '_crp_related_product_tags', true);

        $related = $this->get_product_tag_ids($related_tag_ids,$related);

        if ( ! empty( $related ) && is_array( $related ) ) {
            $related = implode( ',', array_map( 'absint', $related ) );
            $query['where'] .= " AND p.ID IN ( {$related} )";
        }
        return $query;
    }

    function get_product_category_ids($related_categories_ids, $related=array()){

        if(!empty($related_categories_ids)){
    
            foreach($related_categories_ids as $related_categories_id){
    
                $all_ids = get_posts( array(
                    'post_type'     => 'product',
                    'numberposts'   => -1,
                    'post_status'   => 'publish',
                    'fields'        => 'ids',
                    'tax_query'     => array(
                       array(
                          'taxonomy' => 'product_cat',
                          'field'    => 'term_id',
                          'terms'    => $related_categories_id,
                          'operator' => 'IN',
                       )
                    ),
                ) );
    
                if(!empty($related)){
                    $related = array_merge($all_ids,$related);
                }else{
                    $related = $all_ids;
                }
            }
        }
        return $related;
    
    }
	
	    function get_product_tag_ids($related_tag_ids, $related=array()){

        if(!empty($related_tag_ids) && is_array($related_tag_ids)){
    
            foreach($related_tag_ids as $related_tag_id){
    
                $all_ids = get_posts( array(
                    'post_type'     => 'product',
                    'numberposts'   => -1,
                    'post_status'   => 'publish',
                    'fields'        => 'ids',
                    'tax_query'     => array(
                       array(
                          'taxonomy' => 'product_tag',
                          'field'    => 'term_id',
                          'terms'    => $related_tag_id,
                          'operator' => 'IN',
                       )
                    ),
                ) );
    
                if(!empty($related)){
                    $related = array_merge($all_ids,$related);
                }else{
                    $related = $all_ids;
                }
            }
        }
        return $related;
    
    }
    
    public function wt_process_slider_script() {
        $slider_type = get_option('custom_related_products_slider_type') ? get_option('custom_related_products_slider_type'):'swiper';
        if(in_array('elementor/elementor.php', apply_filters('active_plugins', get_option('active_plugins')))){ 
            $slider_type = 'bx';
        }
        
        
        if(strstr(wp_get_theme()->get('Name'),'Woodmart') || strstr(wp_get_theme()->get('Name'),'Flatsome')){
            $slider_type = 'bx';
        }
        
        if(strstr(wp_get_theme()->get('Name'),'Divi') || strstr(wp_get_theme()->get('Name'),'Avada') || strstr(wp_get_theme()->get('Name'),'BeOnePage')){
            $slider_type = 'swiper';
        }
               
        if(strstr(wp_get_theme()->get('Name'),'Twenty Twenty-One')){
            $slider_type = 'swiper';
        }
        
        $slider_state	 = get_option('custom_related_products_slider','enable');
        if('enable' != $slider_state){
            ?>
                <style>
                    .wt-cart-related-products ul.products{
                        flex-wrap: unset !important;
                    }
                </style>
            <?php
        }

        if ('enable' == $slider_state && $slider_type== 'bx') :
                                        $auto_start_slider = apply_filters('wt_custom_related_products_slider_autostart', false);
                                        $bxslider = 'bxslider';
                                        $slide_width = get_option('custom_related_products_crp_banner_width') ? get_option('custom_related_products_crp_banner_width'): 100;//apply_filters('wt_crp_slide_width', 300);
                                        $slide_width_in_px = ($slide_width/100)*300;
                                        $slider_touch = apply_filters('wt_crp_slider_touch_enabled', true);
                                        $min_slides = class_exists('Custom_Related_Products') ? Custom_Related_Products::wt_get_device_type() : '3';//get_option('custom_related_products_crp_banner_product_width') ? get_option('custom_related_products_crp_banner_product_width'): 3;//apply_filters('wt_crp_min_slides', 3);
                                        $max_slides = apply_filters('wt_crp_max_slides', 3);
//                                        if(count($copy) < $max_slides){
//                                          $max_slides = count($copy);
//                                        }
//                                        if(count($copy) < $min_slides){
//                                          $min_slides = count($copy);
//                                        }
                                ?>

                                        <script>
                                                jQuery(document).ready(function($) {
                                                        $('.bxslider').bxSlider({
                                                                slideWidth: '<?php echo $slide_width_in_px;?>',
                                                                minSlides: '<?php echo $min_slides;?>',
                                                                maxSlides: '<?php echo $max_slides;?>',
                                                                moveSlides:1,
                                                                slideMargin: 10,
                                                                startSlide: 0,
                                                                touchEnabled: '<?php echo $slider_touch;?>',
                                                                auto: '<?php echo $auto_start_slider; ?>',
                                                        });
                                                });
                                        </script>
                                        <style>
                                                ul.products li.product.first {
                                                        clear: none !important;
                                                }
                                                .bx-loading {
                                                        display: none;
                                                }
                                                .wt-related-products .crp-slider {
                                                        display: block !important;
                                                }
                                                <?php 
                                                if( apply_filters('wt_crp_slider_center_align', false) ) {
                                                        ?>
                                                        .wt-related-products .bx-wrapper {
                                                                margin-right: auto;
                                                                margin-left: auto;
                                                        }
                                                        <?php
                                                }
                                                ?>
                                                 .wt-related-products ul.products li.product.last {
                                                    margin-right: 10px !important; 
                                                 }

                                                 .wt-related-products ul.products li.product .woocommerce-loop-product__title {
                                                    text-overflow: ellipsis;
                                                    overflow: hidden;
                                                    white-space: nowrap;
                                                 }
                                                 .wt-related-products.products ul.products{
                                                     justify-content: unset !important;
                                                     flex-wrap: unset !important;
                                                     flex-direction: unset ! important;
                                                 }
                                                 .wt-related-products ul.products li.product{
                                                     display: block !important;
                                                 }
                                                .wt-related-products ul.products li.product{
                                                     margin: 0px 10px 0px 0px !important; 
                                                 }
                                                 .wt-related-products .bx-wrapper{
                                                             /*margin-bottom: unset !important;*/ 
                                                             border: none !important; 
                                                             box-shadow: 0 0 0px #ccc!important ;
                                                 }
                                                 .wt-related-products .bx-wrapper .bx-pager{
                                                     display:none;
                                                 }

                                        </style>
                                <?php endif; ?>
                                <?php
                                
                                if ('enable' == $slider_state && $slider_type== 'swiper') :
                                    $auto_start_slider = apply_filters('wt_custom_related_products_slider_autostart', false);
                                    $bxslider = 'bxslider';
                                    $slide_width = get_option('custom_related_products_crp_banner_width') ? get_option('custom_related_products_crp_banner_width'): 100;//apply_filters('wt_crp_slide_width', 300);
                                    $slider_touch = apply_filters('wt_crp_slider_touch_enabled', true);
                                    $min_slides = class_exists('Custom_Related_Products') ? Custom_Related_Products::wt_get_device_type() : '3';//apply_filters('wt_crp_min_slides', 3);
                                    //$max_slides = apply_filters('wt_crp_max_slides', 3);
//                                    if(count($copy) < $min_slides){
//                                        $slide_width = count($copy) * ($slide_width / $min_slides);
//                                        $min_slides = count($copy);
//                                        $few_slider = true;
//                                    }
                                    $slide_width = $slide_width.'%';

                                    ?>

                                    <script>

                                        jQuery(document).ready(function () {
                                        // Swiper: Slider
                                        new Swiper('.swiper-container', {
                                          loop: true,
                                          nextButton: '.swiper-button-next',
                                          prevButton: '.swiper-button-prev',
                                          slidesPerView: <?php echo $min_slides ?>,
                                          paginationClickable: true,
                                          spaceBetween: 10
                                           });
                                      });
                                    </script>
                                 <style>
                                                ul.products li.product.first {
                                                        clear: none !important;
                                                }
                                               
                                                 .wt-related-products ul.products li.product.last {
                                                    margin-right: 10px !important; 
                                                 }

                                                 .wt-related-products ul.products li.product .woocommerce-loop-product__title {
                                                    text-overflow: ellipsis;
                                                    overflow: hidden;
                                                    white-space: nowrap;
                                                 }
                                                 .wt-related-products.products ul.products{
                                                     justify-content: unset !important;
                                                     flex-wrap: unset !important;
                                                     display: flex !important;
                                                     column-gap: unset !important;
                                                     flex-direction: unset ! important;
                                                 }
                                                 .wt-related-products ul.products li.product{
                                                     display: block !important;
                                                 }
                                                .wt-related-products ul.products li.product{
                                                     margin: 0px 10px 0px 0px !important; 
                                                 }
                                                .wt-related-products .swiper-button-prev,.wt-related-products .swiper-button-next{
                                                    width: 35px !important ;
                                                    height: 45px !important;
                                                    background-size: 45px 35px !important;
                                                }
                                        </style>
                            <?php endif;
        
    }
}